- [Dot files](#orgb9209ad)
- [Algorithms](#org7742a9c)
- [Demons](#org8ea31bc)
  - [Built-in demons](#org9966a8e)
  - [Custom demons](#orgf2216a6)
- [CLI](#org95f9d26)
- [Lutin, `lurette`, and `rdbg`](#orgbedc9f2)
  - [Running interactive sessions with `rdbg`](#org77af7a9)
  - [Running batch sessions with `lurette`](#org4e27195)
- [Releases Notes](#org27e58ac)
- [Installation](#org65dd59d)
  - [From source](#orgc3c7db1)
  - [Via opam (not yet working)](#orgb5b1bb5)

Basically, one needs to provide

1.  a dot file
2.  the algorithms mentionned in the dot file


<a id="orgb9209ad"></a>

# Dot files

Dot files should

1.  follow the [graphviz/dot format](https://en.wikipedia.org/wiki/DOT_(graph_description_language)).
2.  all nodes **must be labelled** by the `algo` field

```dot
graph ring {
 p1 [algo="some_algo.ml"]
 p2 [algo="some_algo.ml"]
 p3 [algo="some_algo.ml"]
 p4 [algo="some_algo.ml"]
 p5 [algo="some_algo.ml"]
 p6 [algo="some_algo.ml"]
 p7 [algo="some_algo.ml"]

 p1 -- p2 -- p3 -- p4 -- p5 -- p6 -- p7 -- p1 
}
```

Of course the `some_algo.ml` file must exist.


<a id="org7742a9c"></a>

# Algorithms

Those algorithms, implemented in `ocaml`, must provide:

1.  a set of variables (registers)
2.  an `enable` fonction that says which actions are enabled (i.e., that can be activated)
3.  a `step` function that executes enabled actions

More precisely, each algorithm should provide 3 functions that must be registred using the string used in dot file algo fields with:

-   `reg_vars`
-   `reg_enable`
-   `reg_step`

which profiles is defined in [algo.mli](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/blob/master/lib/algo/algo.mli)

```ocaml
(* Time-stamp: <modified the 19/05/2019 (at 21:43) by Erwan Jahier> *)

(** Process programmer API *)
type varT = It | Ft | Bt | Et of int | St | Nt | At of varT * int
type vars = (string * varT) list 

type value = I of int | F of float | B of bool  | E of int | S of string
           | N of int  (* neighbor canal number *)
           | A of value array
type local_env = string -> value 
type action = string (* label *)

type neighbor = {
  lenv:  local_env;
  n_vars: vars;
  pid: unit -> string; (* Returns the pid of the neigbhor. This info
                          is not available in all modes (e.g.,
                          anonymous) *)
  reply: unit -> int; 
  (* Returns the channel number that let this neighbor access to the
     content of the process, if the neighbor can access to the
     process.  Returns -1 if the neigbor can not access to the
     process, which can happen in directed graphs only.  This info is
     not available in all modes *)
}

type enable_fun = neighbor list -> local_env -> action list
type step_fun   = neighbor list -> local_env -> action -> local_env

(** Those 3 registering functions must be called! *)
type algo_id = string
val reg_vars : algo_id -> vars -> unit
val reg_enable : algo_id  -> enable_fun -> unit
val reg_step : algo_id  -> step_fun -> unit

(** raised by sasa if one of the function above is not called *)
exception Unregistred of string * string

(** This one is not mandatory.  The initialisation done in the dot
   file have priority over this one.  *)
val reg_init_vars : algo_id -> (neighbor list -> local_env) -> unit

(** Mandatory in custom mode only. *)
val reg_actions : algo_id -> action list -> unit

(** util(s) *)
val value_to_string : value -> string


(**/**)
(** functions below are not part of the API *)
val vart_to_rif_decl: varT -> string -> (string * string) list
val vart_to_rif_string: varT -> string -> string
val verbose_level: int ref


(** the following functions are used by sasa *)
val get_vars   : algo_id -> vars
val get_enable : algo_id -> enable_fun
val get_step   : algo_id -> step_fun 
val get_init_vars  : algo_id -> (string * varT) list -> (neighbor list -> local_env) 
val get_actions   : algo_id -> action list
```

Optionnaly, one can register a function that provides initial values to local variables (`reg_init`). If not such registration is done, local variables will be set according to `init` annotations in the dot file. If no such annotations exists, initial values will be chosen at random.

```dot
graph ring {
 p1 [algo="some_algo.ml" init="v=1" init="x=3"]
 p2 [algo="some_algo.ml"]
 p3 [algo="some_algo.ml"]
 p4 [algo="some_algo.ml"]
 p5 [algo="some_algo.ml"]
 p6 [algo="some_algo.ml"]
 p7 [algo="some_algo.ml"]

 p1 -- p2 -- p3 -- p4 -- p5 -- p6 -- p7 -- p1 
}
```

Algorithms must then be compiled with `ocamlopt -shared` to produce the `.cmxs` files corresponding to the `.ml` mentionned in the dot file algo fields.

```sh
ocamlopt -shared -I +sasa some_algo.ml -o some_algo.cmxs
```

Actions names (`string`) must be registered if one wants to use `sasa` with custom demons (cf below).

Some examples can be found in the [test](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test) directory.


<a id="org8ea31bc"></a>

# Demons


<a id="org9966a8e"></a>

## Built-in demons

Built-in demons are usable via one of the following options:

    --synchronous-demon, -sd
    --central-demon, -cd
    --locally-central-demon, -lcd
    --distributed-demon, -dd
    --custom-demon, -custd


<a id="orgf2216a6"></a>

## Custom demons

In the custom demon mode, the demon is executed outside `sasa`, by a process that communicates via the standard input/output using [RIF](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-5) conventions. More precisely, `sasa` writes on `stdout` a Boolean for each action and each process, that states if the corresponding action is enabled for the current step. Then it reads on `stdin` a Boolean for each action and each process that tells which actions (among the enabled ones) should be activated.

The demon can thus be played by users that read and enter Boolean values.

```shell
cd ../../test/unison
sasa ring.dot --custom-demon
```

It can also by simulated by `lutin` programs via the use of `lurette` (for batch executions) or `rdbg` (for interactive sessions).

Built-in demons can of course be programmed in Lutin. One can generate such demons using the `--gen-lutin-demon` option: `sasa --gen-lutin-demon a_graph.dot`. It can be handy at least to get the demons variables names in the good order if one to write its own demon.

Moreover, for using the custom mode, it is mandatory to register actions (via `Algo.reg_actions`).


<a id="org95f9d26"></a>

# CLI

```shell
sasa --help
```

    usage: sasa [<option>]* <topology>.dot 
    use -h to see the available options.
    
    --synchronous-demon, -sd
                Use a Synchronous demon
    --central-demon, -cd
                Use a Central demon (selects exactly one action)
    --locally-central-demon, -lcd
                Use a Locally Central demon
                (i.e., never activates two neighbors actions in the same step)
    --distributed-demon, -dd
                Use a Distributed demon (which select at least one action)
    --custom-demon, -custd
                Use a Custom demon (forces --rif)
    --rif, -rif Follows RIF conventions
    --seed, -seed
                Set the pseudo-random generator seed of build-in demons
    --length, -l <int>
                Maximum number of steps to be done (10000 by default).
    
    --version, -version, -v
                Display the sasa version and exit.
    --verbose, -vl <int>
                Set the verbose level
    --help, -help, -h
                Display main options
    --more, -m  Display more options

For more options:

```shell
sasa --more
```

    --gen-lutin-demon, -gld
                Generate Lutin demons and exit
    --list-algos, -algo
                Output the algo names used in the dot file and exit. 
    --dummy-input
                Add a dummy input to sasa so that built-in demon can be used from rdbg
    --ignore-first-inputs, -ifi
                Ignore first inputs (necessary to use luciole via 
                lurette/rdbg/luciole-rif)
    --ocaml-version
                Display the version ocaml version sasa was compiled with and exit.


<a id="orgbedc9f2"></a>

# Lutin, `lurette`, and `rdbg`

cf `Makefile` in the [test](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test) directory.

In each sub-directory of [../../test/](../../test/), there is a `rdbg-session.ml` file that takes care of the red-tape for using `rdbg` with the corresponding examples.


<a id="org77af7a9"></a>

## Running interactive sessions with `rdbg`

1.  type `rdbg`
2.  press enter to load the defaut session (`rdbg-session.ml`). Then you can type:
3.  `d` for displaying a (dynamic) dot graph with `dot`
4.  or ALTERNATIVELY `ne` to use the `neato` layout engine (`tw`, `ci`, `fd`, `sf`, `pa`, `os` to use the `twopi`, `circo`, `fdp`, `sfd`, `patchwork`, `osage` layout engines respectively)
5.  `s` to move one step forward
6.  `sd` to move one step forward and call `d` (to update the graph display [])
7.  `nr` to move one round forward
8.  `pr` to move one round backward
9.  `go` to move forward until convergence

All those commands are defined in [../../test/my-rdbg-tuning.ml](../../test/my-rdbg-tuning.ml) that is included in `test/*/rdbg-session.ml` files. This file contain straigthforward `ocaml` code that defines various `rdbg` shortcuts to ease the simulation of `sasa` systems. Feel free to tailor those command to yours needs!

For instance, go to the `test/alea-coloring/` directory. It contains the following files:

-   `my-rdbg-tuning.ml` that is actually a symbolic link to the [../../test/my-rdbg-tuning.ml](../../test/my-rdbg-tuning.ml) file
-   `ring.dot`: the topology
-   `p.ml`: the algo used in `ring.dot`
-   `Makefile`: demonstrate various ways of using `sasa`

```sh
make rdbg
```

This rule

-   generates the `p.cmxs` and `p.cma` files, resulting from the compilation of `p.ml` (the makefile rules are in `../Makefile.inc`).
-   generates a `ring.lut` file (out of `ring.dot` and `p.cmxs`) that defines various Lutin demons
-   launch `rdbg` with some arguments. `rdbg` then prompts the user to enter some commands.

```sh
rdbg -o ring.rif  \
      -env "sasa -seed 42  -l 100 ring.dot -custd -rif" \
      -sut-nd "lutin ring.lut -n distributed"

[c] create a fresh session
[q] quit
[s] do nothing

[0/s/q/c/1/2/...]: 
```

hence let's create a fresh session by typing 'c' and then 'return'. This will result in the creation of the `rdbg_session.ml` file, that load all the necessary modules to run an interactive session with `rdbg`, where `rdbg` simulates ring.dot with `sasa` using a distributed demon.

For instance, you can type `nr [return]`, a rdbg command defined in [../../test/my-rdbg-tuning.ml](../../test/my-rdbg-tuning.ml) that steps until the next round is reached. Then, by typing `[return]`; `rdbg` will run the lastly used command, which is `nr`, which allow you to move forward in the execution from rounds to rounds. If you want to go backwards to the previous round, you can use the `br` command=, also defined in [../../test/my-rdbg-tuning.ml](../../test/my-rdbg-tuning.ml).

nb: the next time rdbg is launched (even without argument), you will be prompted with the possibility to reuse this session.

```sh
rdbg

[0] #use "rdbg-session.ml"
[c] create a fresh session
[q] quit
[s] do nothing

[0/s/q/c/1/2/...]: 
```

Typing '0' will therefore also load the `rdbg_session.ml` file we have just been using.


<a id="org4e27195"></a>

## Running batch sessions with `lurette`

XXX TODO


<a id="org27e58ac"></a>

# Releases Notes

<https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/releases>


<a id="org65dd59d"></a>

# Installation


<a id="orgc3c7db1"></a>

## From source

You will need:

-   `make` (gnu)
-   a set of tools installable via [opam](https://opam.ocaml.org/)
    -   `dune`
    -   `ocamlgraph`
    -   `lutin` (not for compiling actually, but for using sasa with custom demons)

One can mimick the content of the `test` job in the project [Gitlab CI script](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/blob/master/.gitlab-ci.yml).


<a id="orgb5b1bb5"></a>

## Via opam (not yet working)

```sh
# optional
opam repo add verimag-sync-repo "http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/opam-repository"
opam update -y
opam depext -y sasa
opam install -y sasa
```

Once this is done, upgrading to the last version of the tools is as simple as:

```yaml
opam update
opam upgrade
```
